MODULE asmlogchlbal_ersem
   !!======================================================================
   !!                       ***  MODULE asmlogchlbal_ersem  ***
   !! Calculate increments to ERSEM based on surface logchl increments
   !!======================================================================
   !! History :  3.6  ! 2016-09  (D. Ford)     Original code
   !!----------------------------------------------------------------------
#if defined key_asminc && defined key_fabm
   !!----------------------------------------------------------------------
   !!   'key_asminc'         : assimilation increment interface
   !!   'key_fabm'           : FABM-ERSEM coupling
   !!----------------------------------------------------------------------
   !!   asm_logchl_bal_ersem : routine to calculate increments to ERSEM
   !!----------------------------------------------------------------------
   USE par_kind,    ONLY: wp             ! kind parameters
   USE par_oce,     ONLY: jpi, jpj, jpk  ! domain array sizes
   USE dom_oce,     ONLY: gdepw_n        ! domain information
   USE zdfmxl                            ! mixed layer depth
   USE iom                               ! i/o
   USE trc,         ONLY: trn            ! ERSEM state variables
   USE par_fabm                          ! FABM parameters

   IMPLICIT NONE
   PRIVATE                   

   PUBLIC asm_logchl_bal_ersem

   REAL(wp), PARAMETER :: tiny = 1e-20

CONTAINS

   SUBROUTINE asm_logchl_bal_ersem( ln_logchlpftinc,          npfts,                    &
      &                             mld_choice_bgc,           logchl_bkginc,            &      
      &                             logchl_balinc_ersem_chl1, logchl_balinc_ersem_chl2, &
      &                             logchl_balinc_ersem_chl3, logchl_balinc_ersem_chl4, &
      &                             logchl_balinc_ersem_p1c,  logchl_balinc_ersem_p1n,  &
      &                             logchl_balinc_ersem_p1p,  logchl_balinc_ersem_p1s,  &
      &                             logchl_balinc_ersem_p2c,  logchl_balinc_ersem_p2n,  &
      &                             logchl_balinc_ersem_p2p,  logchl_balinc_ersem_p3c,  &
      &                             logchl_balinc_ersem_p3n,  logchl_balinc_ersem_p3p,  &
      &                             logchl_balinc_ersem_p4c,  logchl_balinc_ersem_p4n,  &
      &                             logchl_balinc_ersem_p4p,  logchl_balinc_ersem_z4c,  &
      &                             logchl_balinc_ersem_z5c,  logchl_balinc_ersem_z5n,  &
      &                             logchl_balinc_ersem_z5p,  logchl_balinc_ersem_z6c,  &
      &                             logchl_balinc_ersem_z6n,  logchl_balinc_ersem_z6p,  &
      &                             logchl_balinc_ersem_n1p,  logchl_balinc_ersem_n3n,  &
      &                             logchl_balinc_ersem_n4n,  logchl_balinc_ersem_n5s )
      !!---------------------------------------------------------------------------
      !!                    ***  ROUTINE asm_logchl_bal_ersem  ***
      !!
      !! ** Purpose :   calculate increments to ERSEM from logchl increments
      !!
      !! ** Method  :   forthcoming...
      !!
      !! ** Action  :   forthcoming...
      !!
      !! References :   forthcoming...
      !!---------------------------------------------------------------------------
      !!
      LOGICAL,  INTENT(in   )                           :: ln_logchlpftinc
      INTEGER,  INTENT(in   )                           :: npfts
      INTEGER,  INTENT(in   )                           :: mld_choice_bgc
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj,npfts) :: logchl_bkginc
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_chl1
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_chl2
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_chl3
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_chl4
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_p1c
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_p1n
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_p1p
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_p1s
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_p2c
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_p2n
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_p2p
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_p3c
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_p3n
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_p3p
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_p4c
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_p4n
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_p4p
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_z4c
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_z5c
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_z5n
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_z5p
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_z6c
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_z6n
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_z6p
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_n1p
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_n3n
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_n4n
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk)   :: logchl_balinc_ersem_n5s
      !!
      INTEGER                      :: ji, jj, jk
      INTEGER                      :: jkmax
      REAL(wp)                     :: chl_tot, chl_inc
      REAL(wp), DIMENSION(jpi,jpj) :: zmld
      !!---------------------------------------------------------------------------

      ! Split surface logchl incs into surface Chl1-4 incs
      !
      IF ( ln_logchlpftinc ) THEN
         !
         ! Assimilating separate PFTs, so separately transform each from LogChl to Chl
         !
         IF ( npfts /= 4 ) THEN
            CALL ctl_stop( 'If assimilating PFTs into ERSEM, nn_asmpfts must be 4' )
         ENDIF
         DO jj = 1, jpj
            DO ji = 1, jpi
               IF ( trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl1) > 0.0 ) THEN
                  logchl_balinc_ersem_chl1(ji,jj,1) = 10**( LOG10( trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl1) + &
                     &                                             tiny ) + logchl_bkginc(ji,jj,1) ) -    &
                     &                                trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl1) - tiny
               ENDIF
               IF ( trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl2) > 0.0 ) THEN
                  logchl_balinc_ersem_chl2(ji,jj,1) = 10**( LOG10( trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl2) + &
                     &                                             tiny ) + logchl_bkginc(ji,jj,2) ) -    &
                     &                                trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl2) - tiny
               ENDIF
               IF ( trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl3) > 0.0 ) THEN
                  logchl_balinc_ersem_chl3(ji,jj,1) = 10**( LOG10( trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl3) + &
                     &                                             tiny ) + logchl_bkginc(ji,jj,3) ) -    &
                     &                                trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl3) - tiny
               ENDIF
               IF ( trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl4) > 0.0 ) THEN
                  logchl_balinc_ersem_chl4(ji,jj,1) = 10**( LOG10( trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl4) + &
                     &                                             tiny ) + logchl_bkginc(ji,jj,4) ) -    &
                     &                                trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl4) - tiny
               ENDIF
            END DO
         END DO
      ELSE
         !
         ! Assimilating total Chl, so transform total from LogChl to Chl
         ! and split between PFTs according to the existing background ratios
         !
         DO jj = 1, jpj
            DO ji = 1, jpi
               IF ( ( trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl1) > 0.0 ) .AND. &
                  & ( trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl2) > 0.0 ) .AND. &
                  & ( trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl3) > 0.0 ) .AND. &
                  & ( trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl4) > 0.0 ) ) THEN
                  chl_tot = trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl1) + trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl2) + &
                     &      trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl3) + trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl4)
                  chl_inc = 10**( LOG10( chl_tot + tiny ) + logchl_bkginc(ji,jj,1) ) - chl_tot - tiny
                  logchl_balinc_ersem_chl1(ji,jj,1) = chl_inc * trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl1) / chl_tot
                  logchl_balinc_ersem_chl2(ji,jj,1) = chl_inc * trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl2) / chl_tot
                  logchl_balinc_ersem_chl3(ji,jj,1) = chl_inc * trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl3) / chl_tot
                  logchl_balinc_ersem_chl4(ji,jj,1) = chl_inc * trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl4) / chl_tot
               ENDIF
            END DO
         END DO
      ENDIF

      ! Propagate surface Chl1-4 incs through mixed layer
      ! First, choose mixed layer definition
      !
      SELECT CASE( mld_choice_bgc )
      CASE ( 1 )                   ! Turbocline/mixing depth [W points]
         zmld(:,:) = hmld(:,:)
      CASE ( 2 )                   ! Density criterion (0.01 kg/m^3 change from 10m) [W points]
         zmld(:,:) = hmlp(:,:)
      CASE ( 3 )                   ! Kara MLD [Interpolated]
#if defined key_karaml
         IF ( ln_kara ) THEN
            zmld(:,:) = hmld_kara(:,:)
         ELSE
            CALL ctl_stop( ' Kara mixed layer requested for LogChl assimilation,', &
               &           ' but ln_kara=.false.' )
         ENDIF
#else
         CALL ctl_stop( ' Kara mixed layer requested for LogChl assimilation,', &
            &           ' but is not defined' )
#endif
      CASE ( 4 )                   ! Temperature criterion (0.2 K change from surface) [T points]
         zmld(:,:) = hmld_tref(:,:)
      CASE ( 5 )                   ! Density criterion (0.01 kg/m^3 change from 10m) [T points]
         zmld(:,:) = hmlpt(:,:)
      END SELECT
      !
      ! Now set MLD to bottom of a level and propagate incs equally through mixed layer
      !
      DO jj = 1, jpj
         DO ji = 1, jpi
            !
            jkmax = jpk-1
            DO jk = jpk-1, 1, -1
               IF ( ( zmld(ji,jj) >  gdepw_n(ji,jj,jk)   ) .AND. &
                  & ( zmld(ji,jj) <= gdepw_n(ji,jj,jk+1) ) ) THEN
                  zmld(ji,jj) = gdepw_n(ji,jj,jk+1)
                  jkmax = jk
               ENDIF
            END DO
            !
            DO jk = 2, jkmax
               logchl_balinc_ersem_chl1(ji,jj,jk) = logchl_balinc_ersem_chl1(ji,jj,1)
               logchl_balinc_ersem_chl2(ji,jj,jk) = logchl_balinc_ersem_chl2(ji,jj,1)
               logchl_balinc_ersem_chl3(ji,jj,jk) = logchl_balinc_ersem_chl3(ji,jj,1)
               logchl_balinc_ersem_chl4(ji,jj,jk) = logchl_balinc_ersem_chl4(ji,jj,1)
            END DO
            !
         END DO
      END DO

      ! Balance P[1-4][c,n,p,s]
      ! Maintain existing background ratios
      !
      DO jk = 1, jpk
         DO jj = 1, jpj
            DO ji = 1, jpi
               IF ( trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl1) > 0.0 ) THEN
                  logchl_balinc_ersem_p1c(ji,jj,jk) = logchl_balinc_ersem_chl1(ji,jj,jk)   * &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_p1c) / &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_chl1)
                  logchl_balinc_ersem_p1n(ji,jj,jk) = logchl_balinc_ersem_chl1(ji,jj,jk)   * &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_p1n) / &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_chl1)
                  logchl_balinc_ersem_p1p(ji,jj,jk) = logchl_balinc_ersem_chl1(ji,jj,jk)   * &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_p1p) / &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_chl1)
                  logchl_balinc_ersem_p1s(ji,jj,jk) = logchl_balinc_ersem_chl1(ji,jj,jk)   * &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_p1s) / &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_chl1)
               ENDIF
               IF ( trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl2) > 0.0 ) THEN
                  logchl_balinc_ersem_p2c(ji,jj,jk) = logchl_balinc_ersem_chl2(ji,jj,jk)   * &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_p2c) / &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_chl2)
                  logchl_balinc_ersem_p2n(ji,jj,jk) = logchl_balinc_ersem_chl2(ji,jj,jk)   * &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_p2n) / &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_chl2)
                  logchl_balinc_ersem_p2p(ji,jj,jk) = logchl_balinc_ersem_chl2(ji,jj,jk)   * &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_p2p) / &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_chl2)
               ENDIF
               IF ( trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl3) > 0.0 ) THEN
                  logchl_balinc_ersem_p3c(ji,jj,jk) = logchl_balinc_ersem_chl3(ji,jj,jk)   * &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_p3c) / &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_chl3)
                  logchl_balinc_ersem_p3n(ji,jj,jk) = logchl_balinc_ersem_chl3(ji,jj,jk)   * &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_p3n) / &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_chl3)
                  logchl_balinc_ersem_p3p(ji,jj,jk) = logchl_balinc_ersem_chl3(ji,jj,jk)   * &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_p3p) / &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_chl3)
               ENDIF
               IF ( trn(ji,jj,1,jp_fabm_m1+jp_fabm_chl4) > 0.0 ) THEN
                  logchl_balinc_ersem_p4c(ji,jj,jk) = logchl_balinc_ersem_chl4(ji,jj,jk)   * &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_p4c) / &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_chl4)
                  logchl_balinc_ersem_p4n(ji,jj,jk) = logchl_balinc_ersem_chl4(ji,jj,jk)   * &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_p4n) / &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_chl4)
                  logchl_balinc_ersem_p4p(ji,jj,jk) = logchl_balinc_ersem_chl4(ji,jj,jk)   * &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_p4p) / &
                     &                                trn(ji,jj,jk,jp_fabm_m1+jp_fabm_chl4)
               ENDIF
            END DO
         END DO
      END DO

   END SUBROUTINE asm_logchl_bal_ersem

#else
   !!----------------------------------------------------------------------
   !!   Default option : Empty routine
   !!----------------------------------------------------------------------
CONTAINS
   SUBROUTINE asm_logchl_bal_ersem( ln_logchlpftinc,          npfts,                    &
      &                             mld_choice_bgc,           logchl_bkginc,            &      
      &                             logchl_balinc_ersem_chl1, logchl_balinc_ersem_chl2, &
      &                             logchl_balinc_ersem_chl3, logchl_balinc_ersem_chl4, &
      &                             logchl_balinc_ersem_p1c,  logchl_balinc_ersem_p1n,  &
      &                             logchl_balinc_ersem_p1p,  logchl_balinc_ersem_p1s,  &
      &                             logchl_balinc_ersem_p2c,  logchl_balinc_ersem_p2n,  &
      &                             logchl_balinc_ersem_p2p,  logchl_balinc_ersem_p3c,  &
      &                             logchl_balinc_ersem_p3n,  logchl_balinc_ersem_p3p,  &
      &                             logchl_balinc_ersem_p4c,  logchl_balinc_ersem_p4n,  &
      &                             logchl_balinc_ersem_p4p,  logchl_balinc_ersem_z4c,  &
      &                             logchl_balinc_ersem_z5c,  logchl_balinc_ersem_z5n,  &
      &                             logchl_balinc_ersem_z5p,  logchl_balinc_ersem_z6c,  &
      &                             logchl_balinc_ersem_z6n,  logchl_balinc_ersem_z6p,  &
      &                             logchl_balinc_ersem_n1p,  logchl_balinc_ersem_n3n,  &
      &                             logchl_balinc_ersem_n4n,  logchl_balinc_ersem_n5s )
      LOGICAL :: ln_logchlpftinc
      INTEGER :: npfts
      INTEGER :: mld_choice_bgc
      REAL    :: logchl_bkginc(:,:,:)
      REAL    :: logchl_balinc_ersem_chl1(:,:,:), logchl_balinc_ersem_chl2(:,:,:)
      REAL    :: logchl_balinc_ersem_chl3(:,:,:), logchl_balinc_ersem_chl4(:,:,:)
      REAL    :: logchl_balinc_ersem_p1c(:,:,:),  logchl_balinc_ersem_p1n(:,:,:)
      REAL    :: logchl_balinc_ersem_p1p(:,:,:),  logchl_balinc_ersem_p1s(:,:,:)
      REAL    :: logchl_balinc_ersem_p2c(:,:,:),  logchl_balinc_ersem_p2n(:,:,:)
      REAL    :: logchl_balinc_ersem_p2p(:,:,:),  logchl_balinc_ersem_p3c(:,:,:)
      REAL    :: logchl_balinc_ersem_p3n(:,:,:),  logchl_balinc_ersem_p3p(:,:,:)
      REAL    :: logchl_balinc_ersem_p4c(:,:,:),  logchl_balinc_ersem_p4n(:,:,:)
      REAL    :: logchl_balinc_ersem_p4p(:,:,:),  logchl_balinc_ersem_z4c(:,:,:)
      REAL    :: logchl_balinc_ersem_z5c(:,:,:),  logchl_balinc_ersem_z5n(:,:,:)
      REAL    :: logchl_balinc_ersem_z5p(:,:,:),  logchl_balinc_ersem_z6c(:,:,:)
      REAL    :: logchl_balinc_ersem_z6n(:,:,:),  logchl_balinc_ersem_z6p(:,:,:)
      REAL    :: logchl_balinc_ersem_n1p(:,:,:),  logchl_balinc_ersem_n3n(:,:,:)
      REAL    :: logchl_balinc_ersem_n4n(:,:,:),  logchl_balinc_ersem_n5s )
      WRITE(*,*) 'asm_logchl_bal_ersem: You should not have seen this print! error?'
   END SUBROUTINE asm_logchl_bal_ersem
#endif

   !!======================================================================
END MODULE asmlogchlbal_ersem
